!doctype html=""
html lang="en" {
  head {
    meta charset="utf-8"
    meta name="viewport" content="width=device-width, initial-scale=1.0"
    title "{{ status }} | KDL Web Server Test"
    link rel="stylesheet" href="/static/styles/global.css"
  }
  body {
    header class="bg-gray-300" {
      h1 class="text-4xl text-center" "{{ status }}. Oops!"
    }
    main {
      p "{{ error }}"
    }
    footer {
        a href="/" "Back to Home"
    }
  }
}

