!doctype html=""
html lang="en" {
    head {
      meta charset="utf-8"
      title "{% if web_title %}{{ web_title | str_esc }} | {% endif %}KDL Web Server Test"
      link rel="stylesheet" href="./static/styles/global.css"
      {% block head_extra %}{% endblock %}
    }
    body {
      header class="bg-gray-300" {
        h1 class="text-4xl text-center" "{{ header_title | str_esc }}"
        {% block header %}{% endblock %}
      }
      main {
        {% block content %}{% endblock %}
      }
      footer {
        {% include "partials/footer.kdl.j2" %}
      }
    }
}
