.file-list {
  list-style: none;
  padding: 0;
}

.file-list li {
  border-bottom: 1px solid #eee;
}

.file-list a {
  text-decoration: none;
  display: block;
  font-family: monospace;
  font-size: 150%;
  width: 100%;
  padding: 8px 12px;
}

.up-item {
  text-align: center;
  border: 1px solid #40404040;
  margin-bottom: 10px;
  border-radius: 1.5rem;
}

.file-list a.up-button {
  font-size: 150%;
}

.file-list li.directory {
  font-weight: bold;
}

.file-list li:hover {
  background-color: #f0f7ff;
}

.file-list li:active {
  background-color: #a0a4a8;
}