{% extends "layouts/base.kdl.j2" %}

{% set header_title="KDL Web Server Test" %}

{% block content %}
ul {
  li {
    b /*href="/"*/ "Home (you're here!)"
  }
  li {
    a href="listinvoices.html" "View Sample Invoices Table"
  }
}
{% endblock %}
