nav {
  a class="navbutton" href="/index.html" "Back to Home"
  {% if debug %}
    a class="navbutton" href="/_files/" "Browse Files of Example"
  {% endif %}
}

p class="author" {
  b {
    - "by "
    a href="https://wallabra.neocities.org" "wallabra!"
  }
}

p class="shoutout" {
  - "Made with "
  a href="https://palletsprojects.com/projects/jinja/" "Jinja2"
  - " and "
  a href="https://kdl.dev/" "KDL"
  b " <3"
}
