:root {
  --bg-color: #f4f4f9;
  --text-color: #333;
  --accent-color: #7b00ff;
  --border-color: #ddd;
}

body {
  margin: 0;
  padding: 0;
  background-color: var(--bg-color);
  color: var(--text-color);
  line-height: 1.6;
  font-family: sans-serif;
  display: flex;
  flex-direction: column;
  height: 100%;
  margin-bottom: 1em;
}

header {
  background-color: #fff;
  border-bottom: 2px solid var(--border-color);
  padding: 0.5rem 1rem;
  padding-top: 1.5rem;
  display: flex;
  flex-direction: column;
  align-items: center;
}

main {
  display: flex;
  flex-direction: column;
  align-items: center;
  justify-content: center;
  gap: 0.4em;
  max-width: clamp(300px, 90%, 1000px);
  margin: 3rem auto;
  padding: 20px 5rem;
  background: white;
  box-shadow: 0 2px 5px rgba(0, 0, 0, 0.1);
  border-radius: 8px;
  flex: 1;
  border-left: 2px solid var(--border-color);
}

@media (max-width: 800px) {
  body {
    font-size: 150%;
  }

  main {
    max-width: 95%;
    width: 95%;
    padding-left: 4px;
    padding-right: 4px;
  }
}

table {
  width: 100%;
  border-collapse: collapse;
  margin-top: 20px;
  border: 3px solid color-mix(in srgb, var(--border-color), transparent 75%);
  /* it's kinda like a box shadow! */
}

th,
td {
  padding: 12px;
  text-align: left;
  border-bottom: 1px solid var(--border-color);
}

th {
  background-color: #f8f9fa;
  font-weight: bold;
}

h1 {
  padding: 10px;
}

ul {
  gap: 0.3em;
}

footer {
  display: flex;
  flex-direction: column;
  align-items: center;
  width: 100%;
  margin: 2rem auto;
  gap: 1em;
}

footer>p {
  margin: 0px;
  font-size: 80%;
}

/* links */
a {
  color: var(--accent-color);
}

a:visited {
  color: color-mix(in oklab, var(--accent-color) 30%, green);
}

a:hover {
  color: color-mix(in oklab, var(--accent-color) 80%, pink 30%);
}

a:visited:hover {
  color: color-mix(in oklab, var(--accent-color) 20%, pink 65%, green 15%);
}

a:active {
  color: color-mix(in oklab, var(--accent-color) 40%, red 100%);
}

a:visited:active {
  color: color-mix(in oklab, var(--accent-color) 20%, black 50%, green 30%);
}

/* navigation bar */
nav {
  display: flex;
  flex-direction: row;
  flex-wrap: wrap;
  align-items: center;
  justify-content: center;
  gap: 1em;
}

.navbutton,
.navbutton:visited {
  flex-shrink: 0;
  display: inline-block;
  padding: 10px 20px;
  background-color: var(--accent-color);
  color: white;
  text-decoration: none;
  border-radius: 5px;
}

.navbutton:hover,
.navbutton:visited:hover {
  background-color: color-mix(in oklab, var(--accent-color) 100%, white 30%);
  color: white;
}

.navbutton:active,
.navbutton:visited:active {
  background-color: color-mix(in oklab, var(--accent-color) 100%, black 60%);
  color: white;
}